@@ -99,6 +99,11 @@ struct MockTranslator
     MOCK_METHOD1(translate, core::trust::Request::Answer(const core::posix::wait::Result&));
 };
 
+struct MockAppInfoResolver : public core::trust::mir::AppInfoResolver
+{
+    MOCK_METHOD1(resolve, core::trust::mir::AppInfo(const std::string&));
+};
+
 std::shared_ptr<MockConnectionVirtualTable> a_mocked_connection_vtable()
 {
     return std::make_shared<testing::NiceMock<MockConnectionVirtualTable>>();
@@ -117,6 +122,11 @@ std::shared_ptr<MockPromptProviderHelper> a_mocked_prompt_provider_calling_bin_f
                     "/bin/false"
                 });
 }
+
+std::shared_ptr<MockAppInfoResolver> a_mocked_app_info_resolver()
+{
+    return std::make_shared<MockAppInfoResolver>();
+}
 }
 
 TEST(DefaultProcessStateTranslator, throws_for_signalled_process)
@@ -172,7 +182,11 @@ TEST(DefaultPromptProviderHelper, correctly_passes_arguments_to_prompt_executabl
     core::trust::mir::PromptProviderHelper::InvocationArguments iargs
     {
         42,
-        "does.not.exist.application",
+        {
+            "/tmp",
+            "Does not exist",
+            "does.not.exist.application"
+        },
         "Just an extended description for %1%"
     };
 
@@ -193,6 +207,8 @@ TEST(MirAgent, creates_prompt_session_and_execs_helper_with_preauthenticated_fd)
     using namespace ::testing;
 
     const core::trust::Pid app_pid {21};
+    const std::string app_icon{"/tmp"};
+    const std::string app_name {"Does not exist"};
     const std::string app_id {"does.not.exist.application"};
     const core::trust::Feature feature{42};
     const std::string app_description {"This is just an extended description %1%"};
@@ -201,7 +217,11 @@ TEST(MirAgent, creates_prompt_session_and_execs_helper_with_preauthenticated_fd)
     const core::trust::mir::PromptProviderHelper::InvocationArguments reference_invocation_args
     {
         pre_authenticated_fd,
-        app_id,
+        {
+            app_icon,
+            app_name,
+            app_id
+        },
         app_description
     };
 
@@ -209,6 +229,7 @@ TEST(MirAgent, creates_prompt_session_and_execs_helper_with_preauthenticated_fd)
     auto prompt_session_vtable = a_mocked_prompt_session_vtable();
 
     auto prompt_provider_exec_helper = a_mocked_prompt_provider_calling_bin_false();
+    auto app_info_resolver = a_mocked_app_info_resolver();
 
     ON_CALL(*connection_vtable, create_prompt_session_sync(_, _, _))
             .WillByDefault(Return(prompt_session_vtable));
@@ -219,6 +240,9 @@ TEST(MirAgent, creates_prompt_session_and_execs_helper_with_preauthenticated_fd)
     ON_CALL(*prompt_session_vtable, add_prompt_provider_sync(_))
             .WillByDefault(Return(true));
 
+    ON_CALL(*app_info_resolver, resolve(_))
+            .WillByDefault(Return(core::trust::mir::AppInfo{app_icon, app_name, app_id}));
+
     EXPECT_CALL(*connection_vtable, create_prompt_session_sync(app_pid, _, _)).Times(1);
     EXPECT_CALL(*prompt_session_vtable, new_fd_for_prompt_provider()).Times(1);
 
@@ -228,9 +252,13 @@ TEST(MirAgent, creates_prompt_session_and_execs_helper_with_preauthenticated_fd)
 
     core::trust::mir::Agent agent
     {
-        connection_vtable,
-        prompt_provider_exec_helper,
-        core::trust::mir::Agent::translator_only_accepting_exit_status_success()
+        core::trust::mir::Agent::Configuration
+        {
+            connection_vtable,
+            prompt_provider_exec_helper,
+            core::trust::mir::Agent::translator_only_accepting_exit_status_success(),
+            app_info_resolver
+        }
     };
 
     EXPECT_EQ(core::trust::Request::Answer::denied, // /bin/false exits with failure.
@@ -245,11 +273,82 @@ TEST(MirAgent, creates_prompt_session_and_execs_helper_with_preauthenticated_fd)
                   }));
 }
 
+TEST(MirAgent, calls_into_app_info_resolver_with_app_id)
+{
+    using namespace ::testing;
+
+    const core::trust::Pid app_pid {21};
+    const std::string app_id {"does.not.exist.application"};
+    const std::string app_icon{"/tmp"};
+    const std::string app_name{"MeMyselfAndI"};
+    const core::trust::Feature feature{42};
+    const std::string app_description {"This is just an extended description for %1%"};
+    const int pre_authenticated_fd {42};
+
+    const core::trust::mir::PromptProviderHelper::InvocationArguments reference_invocation_args
+    {
+        pre_authenticated_fd,
+        {
+            app_icon,
+            app_name,
+            app_id
+        },
+        app_description
+    };
+
+    auto connection_vtable = a_mocked_connection_vtable();
+    auto prompt_session_vtable = a_mocked_prompt_session_vtable();
+
+    auto prompt_provider_exec_helper = a_mocked_prompt_provider_calling_bin_false();
+    auto app_info_resolver = a_mocked_app_info_resolver();
+
+    ON_CALL(*connection_vtable, create_prompt_session_sync(_, _, _))
+            .WillByDefault(Return(prompt_session_vtable));
+
+    ON_CALL(*prompt_session_vtable, new_fd_for_prompt_provider())
+            .WillByDefault(Return(pre_authenticated_fd));
+
+    ON_CALL(*prompt_session_vtable, add_prompt_provider_sync(_))
+            .WillByDefault(Return(true));
+
+    EXPECT_CALL(*app_info_resolver, resolve(app_id)).Times(1)
+            .WillRepeatedly(
+                Return(core::trust::mir::AppInfo{app_icon, app_name, app_id}));
+    EXPECT_CALL(*prompt_provider_exec_helper,
+                exec_prompt_provider_with_arguments(
+                    reference_invocation_args)).Times(1);
+
+    core::trust::mir::Agent agent
+    {
+        core::trust::mir::Agent::Configuration
+        {
+            connection_vtable,
+            prompt_provider_exec_helper,
+            core::trust::mir::Agent::translator_only_accepting_exit_status_success(),
+            app_info_resolver
+        }
+    };
+
+    EXPECT_EQ(core::trust::Request::Answer::denied, // /bin/false exits with failure.
+              agent.authenticate_request_with_parameters(
+                  core::trust::Agent::RequestParameters
+                  {
+                     core::trust::Uid{::getuid()},
+                     app_pid,
+                     app_id,
+                     feature,
+                     app_description
+                  }));
+
+}
+
 TEST(MirAgent, sig_kills_prompt_provider_process_on_status_change)
 {
     using namespace ::testing;
 
     const core::trust::Pid app_pid {21};
+    const std::string app_icon{"/tmp"};
+    const std::string app_name{"MeMyselfAndI"};
     const std::string app_id {"does.not.exist.application"};
     const core::trust::Feature feature{42};
     const std::string app_description {"This is just an extended description for %1%"};
@@ -270,6 +369,8 @@ TEST(MirAgent, sig_kills_prompt_provider_process_on_status_change)
     auto prompt_provider_helper = std::make_shared<MockPromptProviderHelper>(
                 core::trust::mir::PromptProviderHelper::CreationArguments{"/bin/false"});
 
+    auto app_info_resolver = a_mocked_app_info_resolver();
+
     void* prompt_session_state_callback_context{nullptr};
 
     ON_CALL(*prompt_provider_helper, exec_prompt_provider_with_arguments(_))
@@ -289,6 +390,9 @@ TEST(MirAgent, sig_kills_prompt_provider_process_on_status_change)
                 Return(
                     true));
 
+    ON_CALL(*app_info_resolver, resolve(_))
+            .WillByDefault(Return(core::trust::mir::AppInfo{app_icon, app_name, app_id}));
+
     // An invocation results in a session being created. In addition,
     // we store pointers to callback and context provided by the implementation
     // for being able to later trigger the callback.
@@ -300,9 +404,13 @@ TEST(MirAgent, sig_kills_prompt_provider_process_on_status_change)
 
     core::trust::mir::Agent agent
     {
-        connection_vtable,
-        prompt_provider_helper,
-        core::trust::mir::Agent::translator_only_accepting_exit_status_success()
+        core::trust::mir::Agent::Configuration
+        {
+            connection_vtable,
+            prompt_provider_helper,
+            core::trust::mir::Agent::translator_only_accepting_exit_status_success(),
+            app_info_resolver
+        }
     };
 
     std::thread asynchronously_stop_the_prompting_session
@@ -368,6 +476,8 @@ TEST(TrustPrompt, aborts_for_missing_title)
 #include <core/trust/mir/config.h>
 #include <core/trust/mir/prompt_main.h>
 
+#include <xdg.h>
+
 namespace
 {
 std::map<std::string, std::string> a_copy_of_the_env()
@@ -383,15 +493,25 @@ std::map<std::string, std::string> a_copy_of_the_env()
 std::string mir_socket()
 {
     // We either take the XDG_RUNTIME_DIR or fall back to /tmp if XDG_RUNTIME_DIR is not set.
-    std::string dir = core::posix::this_process::env::get("XDG_RUNTIME_DIR", "/tmp");
-    return dir + "/mir_socket";
+    try
+    {
+        return xdg::runtime().dir().string() + "/mir_socket";
+    } catch(...)
+    {
+        return "/tmp/mir_socket";
+    }
 }
 
 std::string trusted_mir_socket()
 {
     // We either take the XDG_RUNTIME_DIR or fall back to /tmp if XDG_RUNTIME_DIR is not set.
-    std::string dir = core::posix::this_process::env::get("XDG_RUNTIME_DIR", "/tmp");
-    return dir + "/mir_socket_trusted";
+    try
+    {
+        return xdg::runtime().dir().string() + "/mir_socket_trusted";
+    } catch(...)
+    {
+        return "/tmp/mir_socket_trusted";
+    }
 }
 }
 
@@ -404,7 +524,9 @@ TEST(MirAgent, default_agent_works_correctly_against_running_mir_instance_requir
     std::vector<std::string> argv
     {
         "--" + std::string{core::trust::mir::cli::option_server_socket} + "=" + mir_socket(),
-        "--" + std::string{core::trust::mir::cli::option_title} + "=" + pretty_function,
+        "--" + std::string{core::trust::mir::cli::option_icon}, pretty_function,
+        "--" + std::string{core::trust::mir::cli::option_name}, pretty_function,
+        "--" + std::string{core::trust::mir::cli::option_id}, pretty_function,
         "--" + std::string{core::trust::mir::cli::option_description} + "=" + pretty_function,
         // We have to circumvent unity8's authentication mechanism and just provide
         // the desktop_file_hint as part of the command line.
